#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2010 at onward Robert Muth <robert @ muth dot org>.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

import logging
import base64
import re

def _GetUserAndPassword(request):
    auth = request.headers.getheader('Authorization')
    if not auth:
      return None, None

    auth_tokens = auth.split()

    if auth_tokens[0].lower() != 'basic':
      logging.info("wrong authentication type")
      return None, None

    return  base64.decodestring(auth_tokens[-1]).split(':')




def _HasAuthorization(path, permissions):
    """FIXME"""
    if ".." in path:
        return False

    for p in permissions:
        # not zip truncates to the size of the shorter list
        for a, b in zip(p, path):
            if not re.match(a, b):
                break
        else:
            return True
    return False


def GetProfile(request, acl, path):
    """FIXME"""
    if not acl.enable:
        return "default", request.server.config.acl.users["default"]

    user, password = _GetUserAndPassword(request)

    if not user or not password:
        logging.info("no authentication")
        request.SendAuthenticationRequest()
        return None, None

    if user not in acl.users:
        logging.info("unknown user")
        request.SendAuthenticationRequest()
        return None, None

    profile = acl.users[user]
    if profile["pw"] and profile["pw"] != password:
        logging.info("bad password for %s", user)
        request.SendAuthenticationRequest()
        return None, None

    if not _HasAuthorization(path, profile["permissions"]):
        logging.info("no authorization")
        request.SendAuthenticationRequest()
        return None, None


    return user, profile

