#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2010 at onward Robert Muth <robert @ muth dot org>.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================


"""fieda main module"""

# python importd
import SocketServer
import BaseHTTPServer
import sys
import optparse
import os
import socket
import logging

# local imports
import config_helper
import request_handler
import handler_media
import templates
import converter

######################################################################
##
######################################################################

def RequesterString(self):
    host, port = self.client_address[:2]
    # do not use the more expensive: socket.getfqdn(host)
    return '%s' % host

BaseHTTPServer.BaseHTTPRequestHandler.address_string = RequesterString


######################################################################
##
######################################################################
class Server(SocketServer.ThreadingMixIn, BaseHTTPServer.HTTPServer):

  def Reset(self, config):
    self.userLog = [ ] # to track server usage
    self.userIPs = { } # log unique IPs

    self.config = config
    self.resource_dir = config.server.resource_path

    # MEDIA
    self.dirs = []
    if not config.media.dirs:
        logging.fatal(
            "No music directories specified.\n"
            "Please add entries to file frieda.conf, field media.dirs\n")
        sys.exit(-1)
    handler_media.ResetServeDirs()
    for tag, directory in config.media.dirs:
        logging.info("adding media dirs %s -> %s", tag, directory)
        handler_media.AddServeDir(directory, tag)

    templates.Init(config.templates)

  def __init__(self, config):
    self.Reset(config)
    # TODO: make this resetable
    logging.info("convert cache dir is: " + config.server.conversion_cache_path)
    converter.Init(config.server.conversion_cache_path,
                   config.server.conversion_cache_clear,
                   4)
    try:
        SocketServer.TCPServer.__init__(
          self,
          (config.server.binding_hostname, config.server.port),
          request_handler.RequestHandler)
    except socket.error, value:
        logging.error("socket bind failed: %s", str(value[1]))
        raise SystemExit

  def StopServer(self):
      self.shutdown()

  def server_bind(self):
    # set SO_REUSEADDR (if available on this platform)
    if hasattr(socket, 'SOL_SOCKET') and hasattr(socket, 'SO_REUSEADDR'):
      self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)

    BaseHTTPServer.HTTPServer.server_bind(self)


# ======================================================================
# Commandline Parsing
# ======================================================================
parser = optparse.OptionParser(usage="usage: frieda [options]*")

parser.add_option("", "--config",
                  dest="config_file",
                  default = 'frieda.conf',
                  action="store",
                  type="string",
                  help="read config from FILE",
                  metavar="FILE")

parser.add_option("", "--daemon",
                  dest="daemon_mode",
                  default=False,
                  action="store_true",
                  help="daemoninze server")


# ======================================================================
# Commandline Parsing
# ======================================================================


if __name__ == '__main__':

    options, args = parser.parse_args()
    daemon_mode = 0
    config_helper.LoadConfig(options.config_file)

    logformat=("%(levelname)s:"
               "%(module)s:"
               "%(lineno)d:"
               #             "%(threadName)s:"
               " %(message)s")


    config = config_helper.GetConfig()
    logging.basicConfig(level=logging.INFO,
                        format=logformat)
    if config.server.log != "-":
        logging.basicConfig(filemode="w+", filename=config.server.log)
    logging.info("=" * 50)
    logging.info("starting server on port %s", config.server.port)
    logging.info("=" * 50)

    if options.daemon_mode:
        logging.info("daemonizing")
        import daemon
        daemon.daemonize('/dev/null',
                         '/var/log/frieda.log',
                         '/var/log/frieda.log',
                         '/var/run/frieda.pid')


    svr = Server(config)
    logging.info("point your browser at http://localhost:%d", config.server.port)
    svr.serve_forever()
    sys.exit(0)

