#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2008 and onwards Robert Muth <robert at muth dot org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

import logging
import time

import helper
import templates


######################################################################
##
######################################################################
class _datablob:
    def __init__(self, **args):
        self.__dict__.update(args)

COUNT_STATS = {}
USER_LOG = []

def IncCounter(family, name,value=1):
    global COUNT_STATS
    key = (family, name)
    COUNT_STATS[key] = value + COUNT_STATS.get(key, 0)
    return

def LogUser(ip, timestamp, message):
    global USER_LOG
    if len(USER_LOG) > 50:
        USER_LOG.pop(0)
    USER_LOG.append((ip, timestamp, message))
    IncCounter("ip-access", ip)
    return


def MakeUserBlob(ip, timestamp, url):
    d = _datablob()
    d.ip = ip
    d.url = url
    d.unquoted_url = helper.UnquotePath(url)
    d.time = time.strftime("%B %d %I:%M:%S %p", time.localtime(timestamp))
    return d

def RenderUserLog():
    global USER_LOG
    lst = []
    for i in range(len(USER_LOG) - 1, -1, -1):
        ip, timestamp, url = USER_LOG[i]
        lst.append(MakeUserBlob(ip, timestamp, url))
    return lst


def MakeStatBlob(family, name, count):
    d = _datablob()
    d.family = family
    d.name = name
    d.count = count
    return d

def RenderStats():
    global COUNT_STATS
    lst = []
    keys = sorted(COUNT_STATS.keys())
    last_family = None
    for k in keys:
        if k[0] != last_family:
            lst.append(MakeStatBlob(k[0], "", ""))
            last_family = k[0]
        lst.append(MakeStatBlob("", k[1], COUNT_STATS[k]))
    return lst


def MakeLogBlob(timestamp, kind, filename, message):
    d = _datablob()
    d.timestamp = timestamp
    d.kind = kind
    d.filename = helper.EscapeContents(filename)
    d.message = helper.EscapeContents(message)
    return d


def RenderLogs(filename):
    if filename == "-":
        return []

    lst = []
    for line in  open(filename, "r"):
        token = line.split(":", 3)
        if len(token) != 4:
          print token
          continue
        d = MakeLogBlob("",
                        token[0],
                        token[1] + ":" + token[2],
                        token[3])
        lst.append(d)
    return lst[-500:]


def HandlerStats(request, path, args, config, settings):
    template = templates.GetTemplate("stats.ezt")
    request.send_response(200)
    request.send_header("Content-Type", 'text/html')
    request.end_headers()

    data = { 'users' : RenderUserLog(),
             'stats' : RenderStats(),
             'logs'  : RenderLogs(config.server.log),
             }

    template.generate(request.wfile, data)
    return













