#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2008 at onward Robert Muth <robert @ muth dot org>.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

"""frieda mime type module"""

import os


media_extensions = {
    '.mp3' : 'audio/mpeg',
    '.mpc' : 'audio/x-mpc',
    '.flac': 'audio/x-flac',

    '.wav':  'audio/x-wav',
    '.mid' : 'audio/mid',
    '.ogg' : 'audio/ogg',
    '.m4a' : 'audio/mp4',

    '.it'  : 'audio/mid',
    '.xm'  : 'audio/mid',
    '.s3m' : 'audio/mid',
    '.stm' : 'audio/mid',
    '.mod' : 'audio/mid',
    '.dsm' : 'audio/mid',
    '.far' : 'audio/mid',
    '.ult' : 'audio/mid',
    '.mtm' : 'audio/mid',
    '.669' : 'audio/mid',

    '.asx' : 'video/x-ms-asf',
    '.avi' : 'video/x-msvideo',
    '.mpg' : 'video/mpeg',
    '.mp4' : 'video/mp4',
    }

picture_extensions = {
    '.gif'  : 'image/gif',
    '.jpeg' : 'image/jpeg',
    '.jpg'  : 'image/jpeg',
    '.png'  : 'image/png',
    '.ico'  : 'image/png',  # favicon
    }

# Extensions of non-streamed,
# non-media files we want to serve: (and their MIME type)
plainfiles_extensions = {
    '.txt' : 'text/plain',
    '.html' : 'text/html',
    }

playlist_extensions = {
    '.m3u' : 'audio/x-mpegurl',
    '.xspf' : 'application/xspf+xml',
    }

misc_extensions = {
    '.swf' : 'application/x-shockwave-flash',
    '.js'  : 'application/x-javascript',
    '.zip' : 'application/zip'
    }

any_extensions = {}
any_extensions.update(media_extensions)
any_extensions.update(picture_extensions)
any_extensions.update(plainfiles_extensions)
any_extensions.update(playlist_extensions)
any_extensions.update(misc_extensions)

def FileIsLikelyPicture(filename):
    base, ext = os.path.splitext(filename)
    return picture_extensions.has_key(ext.lower())

def FileIsLikelyPlaylist(filename):
    base, ext = os.path.splitext(filename)
    return playlist_extensions.has_key(ext.lower())

def FileIsLikelyPlaintext(filename):
    base, ext = os.path.splitext(filename)
    return plainfiles_extensions.has_key(ext.lower())

def FileIsLikelySong(filename):
    base, ext = os.path.splitext(filename)
    return media_extensions.has_key(ext.lower())

def MimeTypeByExtension(filename):
    base, ext = os.path.splitext(filename)
    return any_extensions[ext.lower()]

