#!/bin/bash
set -o nounset
set -o errexit

GOLDEN=test_golden
TMP=tmp_test
SERVER=localhost:11111


Curl() {
    local url=$1
    local output=$2
    echo "CURL ${url}"
    curl  "${url}" --dump-header ${TMP}/${output}.header > ${TMP}/${output} 2>${TMP}/${output}.log
}


FilterHeader() {
    egrep -v "^(Date|Last-Modified):" $1
}


CheckHeader() {
    output=$1
    echo "DIFF-HEADER  ${TMP}/${output}.header ${GOLDEN}/${output}.header"
    diff  <(FilterHeader ${TMP}/${output}.header) <(FilterHeader ${GOLDEN}/${output}.header)
}

CheckText() {
    output=$1
    echo "DIFF-TEXT  ${TMP}/${output} ${GOLDEN}/${output}"
    diff  ${TMP}/${output}  ${GOLDEN}/${output}
}

CheckDataExact() {
    output=$1
    echo "CMP  ${TMP}/${output} ${GOLDEN}/${output}"
    cmp -l  ${TMP}/${output}  ${GOLDEN}/${output}
}


FilterMedia() {
    file --brief $1
}

FilterTags() {
    ./fileinfo.py $1
}

CheckMedia() {
    output=$1
    echo "MEDIA-CHECK1  ${TMP}/${output} ${GOLDEN}/${output}"
    diff  <(FilterMedia ${TMP}/${output}) <(FilterMedia ${GOLDEN}/${output})
    echo "MEDIA-CHECK2  ${TMP}/${output} ${GOLDEN}/${output}"
    diff  <(FilterTags ${TMP}/${output}) <(FilterTags ${GOLDEN}/${output})
}



RunTests() {
  for cmedia in .mp3 .ogg ; do
      for cplaylist in .m3u .xspf ; do
	  url="${SERVER}/pseudo/TestMusic?cmedia=${cmedia}&cplaylist=${cplaylist}&extinfo"
	  output=pseudo${cplaylist}${cmedia}
	  Curl "${url}" "${output}"
	  CheckHeader "${output}"
	  CheckText "${output}"
	  echo
      done
  done

  for song in pop_mp3.mp3 pop_mpc.mpc pop_flac.flac pop_ogg.ogg ; do
      for cmedia in .mp3 .ogg ; do
	  url="${SERVER}/media/TestMusic/${song}?cmedia=${cmedia}"
	  output=media.${song}${cmedia}
	  Curl "${url}" "${output}"
	  CheckHeader "${output}"
	  CheckMedia "${output}"
	  echo

	  url="${SERVER}/singleton/TestMusic/${song}?cplaylist=.xspf&cmedia=${cmedia}&extinfo"
	  output=singleton.${song}${cmedia}
	  Curl "${url}" "${output}"
	  CheckHeader "${output}"
	  CheckText "${output}"
	  echo
      done
  done



  url="${SERVER}/resources/favicon.ico"
  output="favicon"
  Curl "${url}" "${output}"
  CheckHeader "${output}"
  CheckDataExact "${output}"

  url="${SERVER}/zip/TestMusic/"
  output="zip"
  Curl "${url}" "${output}"
  CheckHeader "${output}"

  echo "TEST OK"
}

KillServer() {
    url="${SERVER}/quitquitquit/"
    curl  --silent "${url}"
}

StartServer() {
    ./frieda.py --config testdata/test_config.conf >$1 2>$2 &
    sleep 1
    local pid=$!
    echo "SERVER PID is: ${pid}"
}

######################################################################
######################################################################
rm -rf ${TMP}
mkdir -p ${TMP}

StartServer ${TMP}/server.stdout.log ${TMP}/server.stderr.log

RunTests
KillServer > /dev/null
KillServer > /dev/null

rm -rf ${TMP}

