#!/usr/bin/env python
# ======================================================================
# Copyright (C) 2010 and onward Robert Muth <robert @ muth dot org>.
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; version 3
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# ======================================================================

# python imports
import atexit
import logging
import shutil
import tempfile
import unittest
#local import
import converter
import fileinfo

INPUT_OGG = "testdata/pop_ogg.ogg"
INPUT_FLAC = "testdata/pop_flac.flac"
INPUT_MP3 = "testdata/pop_mp3.mp3"
INPUT_MPC = "testdata/pop_mpc.mpc"
INPUT_WAV = "testdata/pop_wav.wav"

TEMP_DIR = tempfile.mkdtemp()
converter.Init(TEMP_DIR, 1, 1)
atexit.register(shutil.rmtree, TEMP_DIR)


class TestConvert(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def ValidateTags(self, filename, origin):
        logging.info("checking %s", filename)
        fi = fileinfo.FileInfo(filename, 1)
        self.assertEqual(fi.artist, "artist" + origin)
        self.assertEqual(fi.album, "album" + origin)
        self.assertEqual(fi.title, "title" + origin)
        self.assertEqual(fi.comment, "comment" + origin)
        self.assertEqual(fi.genre, "Cult")
        self.assertEqual(int(fi.track), 6)


    def ConvertAndCheck(self, src, suffix, dst_ext):
        job = converter.SubmitJob(src, dst_ext)
        job.WaitForCompletion()
        self.assertFalse(job.IsFailure())
        self.ValidateTags(job.GetCovertedFile(), suffix)

    def testOGG_2_MP3(self):
        self.ConvertAndCheck(INPUT_OGG, ".ogg", ".mp3")

    def testMPC_2_MP3(self):
        self.ConvertAndCheck(INPUT_MPC, ".mpc", ".mp3")

    def testFLAC_2_MP3(self):
        self.ConvertAndCheck(INPUT_FLAC, ".flac", ".mp3")


    def testMP3_2_OGG(self):
        self.ConvertAndCheck(INPUT_MP3, ".mp3", ".ogg")

    def testMPC_2_OGG(self):
        self.ConvertAndCheck(INPUT_MPC, ".mpc", ".ogg")

    def testFLAC_2_OGG(self):
        self.ConvertAndCheck(INPUT_FLAC, ".flac", ".ogg")


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    unittest.main()

